

	subroutine pgvport2(x1,x2,y1,y2,ix,iy,in,frac,aspect)

! divide normalised device coordinates x1,x2,y1,y2
! into ix by iy seperate sub-divisions
! call pgvport to set device coordinates appropriate
! to n-th (in) sub-division

! the new view port is centered in the middle of each sub-division
! frac set what fraction of the sub-division the new viewport is set to

! aspect determines what aspect ratio (height/width) is required for the plot as
! follows:
! aspect le 0.0 aspect ratio is set by device characteristics
! aspect gt 0.0 aspect ratio is set so that maximum possible plot
! size is set up in sub-division consistent with frac

! This subroutine allows arbitrary portion of devices
! coordinates to be split into ix*iy sub-divisions
! It is a generalisation of PGBEGIN 

! DWM 3-May-95

	real x1,x2,y1,y2,aspect
	integer ix,iy,in
	logical debug

	debug = .false.

	i = mod(in,ix)
	if(i.eq.0) i=ix
	j = 1 + (in-1)/(ix)

	dx = (x2-x1)/float(ix)
	dy = (y2-y1)/float(iy)

	x1vp = (x1 + dx*(float(i)-0.5))-0.5*frac*dx
	x2vp = (x1 + dx*(float(i)-0.5))+0.5*frac*dx

	y1vp = (y2 - dy*(float(j)-0.5))-0.5*frac*dy
	y2vp = (y2 - dy*(float(j)-0.5))+0.5*frac*dy


	if(debug) then
	 write(*,*) 'pgvport2: x1,x2,y1,y2 ',x1,x2,y1,y2
	 write(*,*) 'pgvport2: ix,iy,in,frac ',ix,iy,in,frac
	 write(*,*) 'pgvport2: i,j,dx,dy ',i,j,dx,dy
	 write(*,*) 'pgvport2: x1vp,x2vp,y1vp,y2vp ',x1vp,x2vp,y1vp,y2vp
	end if


	if(aspect.gt.0.0) then

!
! determine correct aspect ratio for plot.
!

	 r0 = aspect ! y/x aspect ratio required


	 call pgvport(x1vp,x2vp,y1vp,y2vp)
!	 call pgwindow(0.0,1.0,0.0,1.0)

	 call pgqvp(2,rx1,rx2,ry1,ry2)


	if(debug) then
	 write(*,*) 'sub-division viewport size in mm: '
	 write(*,*) 'x1: ',rx1
	 write(*,*) 'x2: ',rx2
	 write(*,*) 'y1: ',ry1
	 write(*,*) 'y2: ',ry2
	end if

	   r1 = (ry2-ry1)/(rx2-rx1)

	 if(debug) then
	 write(*,*) 'sub-division aspect ratio: ',r1
	 end if

	 if(r1.gt.r0) then
	   fx = 1.0
	   fy = fx*r0/r1
	 else
	  fy = 1.0
	  fx = fy*r1/r0
	 end if

	x1vp = (x1 + dx*(float(i)-0.5))-0.5*frac*dx*fx
	x2vp = (x1 + dx*(float(i)-0.5))+0.5*frac*dx*fx

	y1vp = (y2 - dy*(float(j)-0.5))-0.5*frac*dy*fy
	y2vp = (y2 - dy*(float(j)-0.5))+0.5*frac*dy*fy

	if(debug) then
	 write(*,*) 'pgvport2: x1vp,x2vp,y1vp,y2vp ',x1vp,x2vp,y1vp,y2vp
	end if

	end if

	if(debug) then
	 write(*,*) 'pgvport2: x1vp,x2vp,y1vp,y2vp ',x1vp,x2vp,y1vp,y2vp
	end if

	call pgvport(x1vp,x2vp,y1vp,y2vp)


        return

	end




