
      subroutine power1(yepoch,mepoch,depoch,jdtoday
     1                 ,asemi,ecc,dist,powergen)



! VSOP solar power generation calculation:
!
!
! use a simple model in which battery power depends on:
!
! (i)   solar power generated which is proportional to inverse
!       square distance from sun
! (ii)  exponential decay of power generation capability with different
!       decay constants at apogee and perigee
! (iii) work has shown that it better to divide 5 year period into two
!       and use different decay constants for each period
! 
! Notes:
! epoch of orbital elements is taken to be BOL
! model works for 5 years after BOL
! model parameters are those that best fit the NEC power curve which
! was given to us in Nov 91
!
!        
!     jdtoday  = Julian date
!     yepoch   = year in which orbital elements defined
!     mepoch   = month in which orbital elements defined
!     depoch   = day of year on which orbital elements defined
!     asemi    = semi-major axis or orbit
!     ecc      = eccentricity or orbit
!     dist     = distance of spacecraft frpm center of earth
!     plifea   = decay timescale (days) of solar panels at apogee
!     plifeb   = decay timescale (days) of solar panels at perigee
!     powera   = power generated (W) at appogee at jdtoday
!     powerp   = power generated (W) at perigee at jdtoday
!     powergen = solar power generated in W at jdtoday
!
! This subroutine was derived from progam /dwm2/fakesatg/power.f
!    DWM 13-DEC-91

	implicit none

	double precision asemi,ecc,dist,powergen,rs,rp
	double precision jdchange,powera,powerp,jdorbit

C-gam	double precision papp,pper,plifea,plifep,r1,r2
	double precision           plifea,plifep,r1,r2  ! papp pper are arrays!!!

	double precision jd1,jd2,jdtoday,papp(3),pper(3)
	double precision ra,dec,lamda,beta,ms
	integer yepoch,mepoch,depoch
	integer idstart,idrange,idchange,idstop


! This calculation is only correct to within a day or so as h:m:s are not used

	 call JUD(depoch,mepoch,yepoch,0,0,0.0d0,jdorbit)
	 call JUD(1,1,yepoch,0,0,0.0d0,jd1)
	 idstart  = idnint(1+jdorbit-jd1)

! If you want a different power curve then just change the following info:


	 idrange   = 1794            ! from NEC power curve
	 idstop   = idstart+idrange
	 idchange = idstart+1057     ! divide power curve at this point
!                                      elapsed day = 1057 from NEC curve

	 papp(1) = 767.0d0 ! appogee power(W) at BOL from NEC curve 
	 pper(1) = 773.0d0 ! perigee power(W) at BOL from NEC curve 
	 papp(2) = 643.0d0 ! appogee power(W) on elapsed day 1057 from NEC curve 
	 pper(2) = 600.0d0 ! perigee power(W) on elapsed day 1057 from NEC curve
	 papp(3) = 587.0d0 ! appogee power(W) on elapsed day 1794 from NEC curve
	 pper(3) = 533.0d0 ! appogee power(W) on elapsed day 1794 from NEC curve


! power calculation:

	 call JUD(idchange,1,yepoch,0,0,0.0d0,jdchange)


	if(jdtoday.le.jdchange) then		

	 call JUD(idstart,1,yepoch,0,0,0.0d0,jd1)
	 call SUN(jd1,ra,dec,lamda,beta,ms,r1)
	 call JUD(idchange,1,yepoch,0,0,0.0d0,jd2)
		
	 call SUN(jd2,ra,dec,lamda,beta,ms,r2)

	 plifea=(jd2-jd1)/dlog((papp(1)/papp(2))*((r1/r2)**2))
	 plifep=(jd2-jd1)/dlog((pper(1)/pper(2))*((r1/r2)**2))

	 call SUN(jdtoday,ra,dec,lamda,beta,ms,rs) 

	 powera=( papp(1)*((r1/rs)**2)*dexp(-(jdtoday-jd1)/plifea) )
	 powerp=( pper(1)*((r1/rs)**2)*dexp(-(jdtoday-jd1)/plifep) )


!	 write(*,*) 'First part of curve '
!	 write(*,*) 'apogee  lifetime (yr) ',plifea/365.25d0
!	 write(*,*) 'perigee lifetime (yr) ',plifep/365.25d0

	else

	 call JUD(idchange,1,yepoch,0,0,0.0d0,jd1)		
	 call SUN(jd1,ra,dec,lamda,beta,ms,r1)
	 call JUD(idstop,1,yepoch,0,0,0.0d0,jd2)		
	 call SUN(jd2,ra,dec,lamda,beta,ms,r2)

	 plifea=(jd2-jd1)/dlog((papp(2)/papp(3))*((r1/r2)**2))
	 plifep=(jd2-jd1)/dlog((pper(2)/pper(3))*((r1/r2)**2))

	 call SUN(jdtoday,ra,dec,lamda,beta,ms,rs) 

	 powera=( papp(2)*((r1/rs)**2)*dexp(-(jdtoday-jd1)/plifea) )
	 powerp=( pper(2)*((r1/rs)**2)*dexp(-(jdtoday-jd1)/plifep) ) 

!	 write(*,*) 'Second part of curve '
!	 write(*,*) 'apogee  lifetime (yr) ',plifea/365.25d0
!	 write(*,*) 'perigee lifetime (yr) ',plifep/365.25d0

	end if

! use simple linear approximation to work out power at any altitude

	ra = (1.0d0+ecc)*asemi
	rp = (1.0d0-ecc)*asemi

	powergen = powerp + (powera-powerp)*((dist-rp)/(ra-rp))

!	write(*,*) (dist-rp)/(ra-rp),power,powera,powerp

	return

	end
