


	subroutine tant_calc(ang1,r,bandname,init,approx,tant)

! given the angle from the pointing direction to the center of the
! earth, ang1, in radians and the distance of the spacecraft from the
! center of the earth, r, in m and a single character, bandname that determines
! the observing band returns the increase in the antenna temperature, tant, in
! K due to the earth and the sky background.

! for each new observing band the logical init should be set to set true
! When init is true the anetnna gain file is read in and various calculations
! are preformed. 

! if the logical approx is true then an approximation to the antenna gain is 
! used

! Note: This subroutine is for VSOP only.

! DWM 31-Aug-94

	real gain2(180),ang(361)
	logical init, approx
	character*80 trash,title,file
	character*1 bandname

	save

! set up ifreq

	if(bandname.eq.'L') ifreq = 1
	if(bandname.eq.'C') ifreq = 2
	if(bandname.eq.'K') ifreq = 3

! set up the file names of antenna gain files

	 if(ifreq.eq.1) file = 'vlbil.dat'
	 if(ifreq.eq.2) file = 'vlbic.dat'
	 if(ifreq.eq.3) file = 'vlbik.dat'


! set up constants

	pi = 3.141592654
	rd = 180.0/pi
	dr = pi/180.0

! earth radius in km and determine s/c height in km

	rekm = 6378.14 

	hkm = (r/1000.0-rekm)

! earth temperature in K

	tearth = 300.0

! number of integration steps to use
! set equal to 180 as gain information from
! ISAS is in steps of 1 degree

	inum = 180

! read in antenna gain information

	if(init) then

	 open(1,file=file, status='old',err=999)

	 read(1,*) tback

	 if(tback.le.0.0) then

! set up sky background temperature, tback, in K
! tback from pg 42 of VSOP-TD-94-006
! ignoring the sun that is v small effect (0.007 K )

	 if(ifreq.eq.1) tback = 13.0
	 if(ifreq.eq.2) tback = 4.0
	 if(ifreq.eq.3) tback = 3.0

	end if

	write(*,*) ' '
	write(*,*) 'using a background temperature (K) of: ',tback
	write(*,*)

	do i=1,4
	read(1,'(a)') trash
	end do

	iostat=0
	ic    = 0

        do while (iostat.eq.0)
	  ic = ic + 1
          read(1,*,iostat=iostat,end=25) ang(ic),gain
	  if(ic.eq.1) gmin = gain
	  if(ic.eq.1) gmax = gain
	  if(gain2(ic).le.gmin) gmin = gain
	  if(gain2(ic).ge.gmax) gmax = gain

	  if(ang(ic).gt.0.0) then
	   iang = int(ang(ic) +0.1)
	   gain2(iang) = 10.0**(gain/10.0)
!	   write(*,*) ifreq,ang(ic),iang,gain2(ic)
!	   write(*,*) ang(ic), iang,gain,gain2(iang)
	  end if

	end do

25	continue

	  ic = ic - 1

	  close(1)

! work out main beam contribution

	if(ifreq.eq.1) write(*,*) 'frequency: 1.6 GHz '
	if(ifreq.eq.2) write(*,*) 'frequency: 5   GHz '
	if(ifreq.eq.3) write(*,*) 'frequency: 22  GHz '

	x1 = 0.0
	x2 = pi
	dx = (x2 - x1)/float(inum)

	gmain = 0.0

!	do i=1,inum
!	 x = 0.5*dx*(2.0*float(i)-1) + x1

! make slight error here:
! only integrating from 0.5 to 179.5

	do i=1,inum-1

	 iang = i
	 x = float(i)*dr

	 call gapprox(bandname,x,gain)
	 if(.not.approx) gain = gain2(iang)

	 gmain = gmain + 2.0*pi*gain*sin(x)*dx
	 gs = 2.0*pi*gain*sin(x)
!	 write(*,*) 'i,x,g,s,gs,gm: ',i,xdeg,gain,sin(x),gs,gmain
	end do
	
	gmain = 4.0*pi - gmain

	write(*,*) 'gmain/4*pi is: ', gmain/(4.0*pi)


	end if !if(init)


! calculate half cone angular size of earth

	ange = asin(1.0/(1.0 + (hkm/rekm)))

! convert this to integer angles

!	x1 = ang1 - ange
!	x2 = ang1 + ange
!	if(x1.lt.0.0) x1 = 0.0
!	if(x2.gt.pi)  x2 = pi
!	dx = (x2 - x1)/float(inum)

	x1 = (ang1 - ange)*rd
	x2 = (ang1 + ange)*rd

	if(x1.lt.0.0)   x1 = 0.0
	if(x2.gt.180.0) x2 = 180.0

	ix1 = int(x1 + 0.1) + 1
	ix2 = int(x2 + 0.1)

!	write(*,*) 'ang1,ange,x1,ix1,x2,ix2: ',ang1*rd,ange*rd,x1,ix1,x2,ix2

	if(ix1.lt.1)    ix1 = 1
	if(ix2.gt.180)  ix2 = 179
	dx = 1.0*dr


!	write(*,*) 'ang1,ange,x1,x2: ',ang1,ange,x1,x2

	tant = 0.0

!	do i=1,inum
!	 x = 0.5*dx*(2.0*float(i)-1) + x1

! make slight error here:
! only integrating from 0.5 to 179.5

	do i=ix1,ix2

	 iang = i
	 x = float(i)*dr

	f1 = cos(x)*cos(ang1)
	f2 = sin(x)*sin(ang1)
	f3 = (cos(ange) - f1)/f2

!	write(*,*) 'x,a1,ange,f1,f2,f3: ',x*rd,ang1*rd,ange*rd,f1,f2,f3

! set full range of ang2 if for a given x all of differential solid
! angle is within the earth

	if(abs(f3).gt.1.0) then	
	 ang2 = pi
	else
	 ang2 = acos(f3)
	end if

	 call gapprox(bandname,x,gain)
	 if(.not.approx) gain = gain2(iang)


	tant = tant + (2.0 * ang2 * gain * sin(x)*dx)

	if(i.eq.ix1)   t1 = 2.0 * ang2 * gain * sin(x)*dx
	if(i.eq.ix1+1) t2 = 2.0 * ang2 * gain * sin(x)*dx
	if(i.eq.ix2)   t3 = 2.0 * ang2 * gain * sin(x)*dx
	if(i.eq.ix2-1) t4 = 2.0 * ang2 * gain * sin(x)*dx

	end do

! Try to correct for end effects in integral. This does not work yet
! effect seems to be small anyway

!	 f1 = (x1-float(ix1))*dr*(t1-t2)/dx
!	 f2 = (x2-float(ix2))*dr*(t3-t4)/dx
!	 f3 = f1 + f2
!	 tant = tant + f1 + f2

! add main beam contribution if required

!	write(*,*) 'ang1,ange: ',ang1*rd,ange*rd


	if(ang1.le.ange) then
!	  write(*,*) 'earth is in the main beam ! '
	  tant = tant + gmain 
	else
!	  write(*,*) 'earth is NOT in the main beam ! '
	end if

! background contribution and temp of earth added here


	tant = (tant * tearth + tback*(4.0*pi-tant))/(4.0*pi)


	init = .false. 

	return

999	write(*,*) '*** error opening gain file:'
	write(*,'(a)') file

	return
	
	end
